@@ -291,7 +291,9 @@ def get_run_metadata(run_id, session=None):
     """
     session = session or get_session()
     query = db_utils.model_query(models.RunMetadata, session).join(
-        models.RunMetadata.run).filter(models.Run.uuid == run_id)
+        models.Run,
+        models.RunMetadata.run_id == models.Run.id).filter(
+            models.Run.uuid == run_id)
     return query.all()
 
 
@@ -307,7 +309,9 @@ def get_runs_by_key_value(key, value, session=None):
     """
     session = session or get_session()
     query = db_utils.model_query(models.Run, session=session).join(
-        models.RunMetadata).filter_by(key=key, value=value)
+        models.RunMetadata,
+        models.Run.id == models.RunMetadata.run_id).filter_by(
+            key=key, value=value)
 
     return query.all()
 
@@ -642,7 +646,7 @@ def get_test_runs_by_test_id(test_id, session=None):
 
 
 def get_test_runs_by_test_test_id(test_id, start_date=None, stop_date=None,
-                                  session=None):
+                                  session=None, key=None, value=None):
     """Get all test runs for a specific test by the test'stest_id column
 
     :param str test_id: The test's test_id (the test_id column in the test
@@ -653,6 +657,12 @@ def get_test_runs_by_test_test_id(test_id, start_date=None, stop_date=None,
                                          results
     :param datetime.datetime stop_date: The date to use as the cutoff date for
                                         results
+    :param str key: an optional key for run metadata to filter the test runs
+                    on. Must be specified with a value otherwise it does
+                    nothing.
+    :param str value: an optional value for run metadata to filter the test
+                      runs on. Must be specified with a key otherwise it does
+                      nothing.
 
     :return list: The list of test run objects for the specified test
     :rtype: subunit2sql.models.TestRun
@@ -661,7 +671,7 @@ def get_test_runs_by_test_test_id(test_id, start_date=None, stop_date=None,
 
     test_runs_query = db_utils.model_query(models.TestRun,
                                            session=session).join(
-        models.Test).filter(
+        models.Test, models.TestRun.test_id == models.Test.id).filter(
             models.Test.test_id == test_id)
     if start_date:
         test_runs_query = test_runs_query.filter(
@@ -669,6 +679,12 @@ def get_test_runs_by_test_test_id(test_id, start_date=None, stop_date=None,
     if stop_date:
         test_runs_query = test_runs_query.filter(
             models.TestRun.start_time <= stop_date)
+    if key and value:
+        test_runs_query = test_runs_query.join(
+            models.RunMetadata,
+            models.TestRun.run_id == models.RunMetadata.run_id).filter(
+                models.RunMetadata.key == key,
+                models.RunMetadata.value == value)
     test_runs = test_runs_query.all()
     return test_runs
 
@@ -687,7 +703,7 @@ def get_test_runs_by_run_id(run_id, session=None):
     session = session or get_session()
     test_runs = db_utils.model_query(
         models.TestRun, session=session).join(
-            models.Run).filter(
+            models.Run, models.TestRun.run_id == models.Run.id).filter(
                 models.Run.uuid == run_id).all()
     return test_runs
 
@@ -728,7 +744,8 @@ def get_tests_from_run_id(run_id, session=None):
     """
     session = session or get_session()
     query = db_utils.model_query(models.Test, session=session).join(
-        models.TestRun).filter_by(run_id=run_id)
+        models.TestRun, models.Test.id == models.TestRun.test_id).filter_by(
+            run_id=run_id)
     return query.all()
 
 
@@ -750,19 +767,22 @@ def get_tests_run_dicts_from_run_id(run_id, session=None):
     """
     session = session or get_session()
     query = db_utils.model_query(models.Test, session=session).join(
-        models.TestRun).join(models.Run).filter(
-            models.Run.uuid == run_id).outerjoin(
-                models.TestRunMetadata).order_by(
-                    models.TestRun.start_time,
-                    models.TestRun.start_time_microsecond).values(
-                        models.Test.test_id,
-                        models.TestRun.status,
+        models.TestRun, models.Test.id == models.TestRun.test_id).join(
+            models.Run, models.TestRun.run_id == models.Run.id).filter(
+                models.Run.uuid == run_id).outerjoin(
+                    models.TestRunMetadata,
+                    models.TestRun.id == models.TestRunMetadata.
+                    test_run_id).order_by(
                         models.TestRun.start_time,
-                        models.TestRun.start_time_microsecond,
-                        models.TestRun.stop_time,
-                        models.TestRun.stop_time_microsecond,
-                        models.TestRunMetadata.key,
-                        models.TestRunMetadata.value)
+                        models.TestRun.start_time_microsecond).values(
+                            models.Test.test_id,
+                            models.TestRun.status,
+                            models.TestRun.start_time,
+                            models.TestRun.start_time_microsecond,
+                            models.TestRun.stop_time,
+                            models.TestRun.stop_time_microsecond,
+                            models.TestRunMetadata.key,
+                            models.TestRunMetadata.value)
     test_runs = collections.OrderedDict()
     for test_run in query:
         if test_run[0] not in test_runs:
@@ -817,20 +837,34 @@ def get_test_run_time_series(test_id, session=None):
     return time_series
 
 
-def get_test_run_series(start_date=None, stop_date=None, session=None):
+def get_test_run_series(start_date=None, stop_date=None, session=None,
+                        key='build_queue', value='gate'):
     """Returns a time series dict of total daily run counts
 
     :param str start_date: Optional start date to filter results on
     :param str stop_date: Optional stop date to filter results on
     :param session: optional session object if one isn't provided a new session
-
+    :param str key: optional run_metadata key to filter the runs used on. Key
+                    must be specified with value for filtering to occur.
+                    This defaults to 'build_queue' for backwards compatibility
+                    with earlier versions. Note, this default will be removed
+                    in the future.
+    :param str value: optional run_metadata value to filter the runs used on.
+                      Value must be specified with key for filtering to occur.
+                      This defaults to 'gate' for backwards
+                      compatibility with earlier versions. Note, this default
+                      will be removed in the future.
     :return dict: A dictionary with the dates as the keys and the values
                   being the total run count for that day. (The sum of success
                   and failures from all runs that started that day)
     """
     session = session or get_session()
-    full_query = db_utils.model_query(models.Run, session=session).join(
-        models.RunMetadata).filter_by(key='build_queue', value='gate')
+    full_query = db_utils.model_query(models.Run, session=session)
+    if key and value:
+        full_query = full_query.join(
+            models.RunMetadata,
+            models.Run.id == models.RunMetadata.run_id).filter_by(
+                key=key, value=value)
 
     # Process date bounds
     full_query = _filter_runs_by_date(full_query, start_date, stop_date)
@@ -904,6 +938,25 @@ def get_recent_failed_runs(num_runs=10, session=None):
     return list(map(lambda x: x.uuid, results))
 
 
+def get_recent_runs_by_key_value_metadata(key, value, num_runs=10,
+                                          session=None):
+    """Get a list of runs for recent runs with a key value metadata pair
+
+    :param int num_runs: The number of runs to return in the list
+    :param session: optional session object if one isn't provided a new session
+    :return list: A list of run objects for the most recent runs.
+    :rtype subunit2sql.db.models.Run
+    """
+    session = session or get_session()
+    results = db_utils.model_query(models.Run, session).join(
+        models.RunMetadata,
+        models.Run.id == models.RunMetadata.run_id).filter(
+            models.RunMetadata.key == key,
+            models.RunMetadata.value == value).order_by(
+                models.Run.run_at.desc()).limit(num_runs).all()
+    return results
+
+
 def delete_old_runs(expire_age=186, session=None):
     """Delete all runs and associated metadata older than the provided age
 
@@ -919,10 +972,10 @@ def delete_old_runs(expire_age=186, session=None):
         models.Run.run_at < expire_date).subquery()
     db_utils.model_query(models.RunMetadata, session).filter(
         models.RunMetadata.run_id.in_(sub_query)).delete(
-            synchronize_session='fetch')
+            synchronize_session=False)
     # Delete the runs
     db_utils.model_query(models.Run, session).filter(
-        models.Run.run_at < expire_date).delete(synchronize_session='evaluate')
+        models.Run.run_at < expire_date).delete(synchronize_session=False)
 
 
 def delete_old_test_runs(expire_age=186, session=None):
@@ -940,17 +993,17 @@ def delete_old_test_runs(expire_age=186, session=None):
         models.TestRun.start_time < expire_date).subquery()
     db_utils.model_query(models.TestRunMetadata, session).filter(
         models.TestRunMetadata.test_run_id.in_(sub_query)).delete(
-            synchronize_session='fetch')
+            synchronize_session=False)
     # Delete the test runs
     db_utils.model_query(models.TestRun, session).filter(
         models.TestRun.start_time < expire_date).delete(
-            synchronize_session='evaluate')
+            synchronize_session=False)
 
 
 def get_id_from_test_id(test_id, session=None):
     """Return the id (uuid primary key) for a test given it's test_id value
 
-    :param str test_id:
+    :param str test_id: The test_id's string (not UUID) to identify the test
     :param session: optional session object if one isn't provided a new session
                     will be acquired for the duration of this operation
     :return: The id for the specified test
@@ -973,6 +1026,39 @@ def get_ids_for_all_tests(session=None):
     return db_utils.model_query(models.Test, session).value(models.Test.id)
 
 
+def get_run_times_grouped_by_run_metadata_key(key, start_date=None,
+                                              stop_date=None, session=None):
+    """Return the aggregate run times for all runs grouped by a metadata key
+
+    :param key: The run_metadata key to use for grouping runs
+    :param session: Optional session object if one isn't provided a new session
+                        will be acquired for the duration of this operation
+
+    :return: A dictionary where keys are the value of the provided metadata key
+             and the values are a list of run_times for successful runs with
+             that metadata value
+    :rtype: dict
+    """
+    session = session or get_session()
+    run_times_query = db_utils.model_query(models.Run, session).filter(
+        models.Run.fails == 0, models.Run.passes > 0).join(
+            models.RunMetadata,
+            models.Run.id == models.RunMetadata.run_id).filter(
+                models.RunMetadata.key == key)
+
+    run_times_query = _filter_runs_by_date(run_times_query, start_date,
+                                           stop_date)
+    run_times = run_times_query.values(models.Run.run_at, models.Run.run_time,
+                                       models.RunMetadata.value)
+    result = {}
+    for run in run_times:
+        if result.get(run.value):
+            result[run.value].append(run.run_time)
+        else:
+            result[run.value] = [run.run_time]
+    return result
+
+
 def get_test_counts_in_date_range(test_id, start_date=None, stop_date=None,
                                   session=None):
     """Return the number of successes, failures, and skips for a single test.
@@ -980,6 +1066,7 @@ def get_test_counts_in_date_range(test_id, start_date=None, stop_date=None,
     Optionally you can provide a date to filter the results to be within a
     certain date range
 
+    :param str test_id: The test_id's ID(big integer) to identify the test
     :param str start_date: The date to use as the start for counting
     :param str stop_date: The date to use as the cutoff for counting
     :param session: optional session object if one isn't provided a new session
@@ -1072,7 +1159,7 @@ def get_test_run_dict_by_run_meta_key_value(key, value, start_date=None,
         models.RunMetadata.value == value).join(
             models.TestRun,
             models.RunMetadata.run_id == models.TestRun.run_id).join(
-                models.Test)
+                models.Test, models.TestRun.test_id == models.Test.id)
     query = _filter_test_runs_by_date(query, start_date=start_date,
                                       stop_date=stop_date)
     query = query.values(models.Test.test_id,
@@ -1083,19 +1170,21 @@ def get_test_run_dict_by_run_meta_key_value(key, value, start_date=None,
                          models.TestRun.stop_time_microsecond)
     tests = []
     for test in query:
-        if test[2]:
-            start_time = test[2]
-            start_time = start_time.replace(microsecond=test[3])
+        if test.start_time:
+            start_time = test.start_time
+            start_time = start_time.replace(
+                microsecond=test.start_time_microsecond)
         else:
             start_time = None
-        if test[4]:
-            stop_time = test[4]
-            stop_time = stop_time.replace(microsecond=test[5])
+        if test.stop_time:
+            stop_time = test.stop_time
+            stop_time = stop_time.replace(
+                microsecond=test.stop_time_microsecond)
         else:
             stop_time = None
         test_run_dict = {
-            'test_id': test[0],
-            'status': test[1],
+            'test_id': test.test_id,
+            'status': test.status,
             'start_time': start_time,
             'stop_time': stop_time,
         }
@@ -1121,7 +1210,9 @@ def get_all_runs_time_series_by_key(key, start_date=None,
     """
     session = session or get_session()
     runs_query = db_utils.model_query(models.Run, session).join(
-        models.RunMetadata).filter(models.RunMetadata.key == key)
+        models.RunMetadata,
+        models.Run.id == models.RunMetadata.run_id).filter(
+            models.RunMetadata.key == key)
     runs_query = _filter_runs_by_date(runs_query, start_date, stop_date)
     runs_query = runs_query.values(models.Run.run_at,
                                    models.Run.passes,
@@ -1130,24 +1221,24 @@ def get_all_runs_time_series_by_key(key, start_date=None,
                                    models.RunMetadata.value)
     runs = {}
     for run in runs_query:
-        if run[0] not in runs:
-            runs[run[0]] = {run[4]: [{
-                'pass': run[1],
-                'fail': run[2],
-                'skip': run[3],
+        if run.run_at not in runs:
+            runs[run.run_at] = {run.value: [{
+                'pass': run.passes,
+                'fail': run.fails,
+                'skip': run.skips,
             }]}
         else:
-            if run[4] not in list(runs[run[0]].keys()):
-                runs[run[0]][run[4]] = [{
-                    'pass': run[1],
-                    'fail': run[2],
-                    'skip': run[3],
+            if run.value not in list(runs[run.run_at].keys()):
+                runs[run.run_at][run.value] = [{
+                    'pass': run.passes,
+                    'fail': run.fails,
+                    'skip': run.skips,
                 }]
             else:
-                runs[run[0]][run[4]].append({
-                    'pass': run[1],
-                    'fail': run[2],
-                    'skip': run[3],
+                runs[run.run_at][run.value].append({
+                    'pass': run.passes,
+                    'fail': run.fails,
+                    'skip': run.skips,
                 })
     return runs
 
@@ -1173,7 +1264,9 @@ def get_time_series_runs_by_key_value(key, value, start_date=None,
         models.RunMetadata.key == key,
         models.RunMetadata.value == value).subquery()
     run_query = db_utils.model_query(models.Run, session).join(
-        models.RunMetadata).filter(models.Run.id.in_(sub_query))
+        models.RunMetadata,
+        models.Run.id == models.RunMetadata.run_id).filter(
+            models.Run.id.in_(sub_query))
     run_query = _filter_runs_by_date(run_query, start_date, stop_date)
     run_query = run_query.values(models.Run.uuid,
                                  models.Run.passes,
@@ -1185,8 +1278,8 @@ def get_time_series_runs_by_key_value(key, value, start_date=None,
                                  models.RunMetadata.value)
     runs = {}
     for run in run_query:
-        run_at = run[5]
-        run_id = run[0]
+        run_at = run.run_at
+        run_id = run.uuid
         if run_at not in runs:
             # We have hit a new time stamp so we need to add a top level key
             # for the timestamp and populate the run list with a new dict for
@@ -1194,11 +1287,11 @@ def get_time_series_runs_by_key_value(key, value, start_date=None,
             runs[run_at] = []
             run_dict = {
                 'id': run_id,
-                'pass': run[1],
-                'fail': run[2],
-                'skip': run[3],
-                'run_time': run[4],
-                'metadata': {run[6]: run[7]}
+                'pass': run.passes,
+                'fail': run.fails,
+                'skip': run.skips,
+                'run_time': run.run_time,
+                'metadata': {run.key: run.value}
             }
             runs[run_at].append(run_dict)
         else:
@@ -1207,11 +1300,11 @@ def get_time_series_runs_by_key_value(key, value, start_date=None,
                 # append a new run dict to the list of runs for that timestamp
                 run_dict = {
                     'id': run_id,
-                    'pass': run[1],
-                    'fail': run[2],
-                    'skip': run[3],
-                    'run_time': run[4],
-                    'metadata': {run[6]: run[7]}
+                    'pass': run.passes,
+                    'fail': run.fails,
+                    'skip': run.skips,
+                    'run_time': run.run_time,
+                    'metadata': {run.key: run.value}
                 }
                 runs[run_at].append(run_dict)
             else:
@@ -1222,10 +1315,40 @@ def get_time_series_runs_by_key_value(key, value, start_date=None,
                 for index, run_dict in list(enumerate(runs[run_at])):
                     if run_dict['id'] == run_id:
                         update_index = index
-                runs[run_at][update_index]['metadata'][run[6]] = run[7]
+                runs[run_at][update_index]['metadata'][run.key] = run.value
     return runs
 
 
+def get_run_failure_rate_by_key_value_metadata(key, value, start_date=None,
+                                               stop_date=None, session=None):
+    """Return the failure percentage of runs with a set of run metadata
+
+    :param str key: the metadata key to use for matching the runs
+    :param str value: the metadata value to use for matching the runs
+    :param start_date: Optional start date to filter results on
+    :param str stop_date: Optional stop date to filter results on
+    :param session: optional session object if one isn't provided a new session
+                    will be acquired for the duration of this operation
+
+    :return failure_rate: The percentage of runs that failed, will be None if
+                          no runs are found
+    :rtype: float
+    """
+    session = session or get_session()
+    base_query = db_utils.model_query(models.Run, session).join(
+        models.RunMetadata,
+        models.Run.id == models.RunMetadata.run_id).filter(
+            models.RunMetadata.key == key,
+            models.RunMetadata.value == value)
+    base_query = _filter_runs_by_date(base_query, start_date, stop_date)
+    fails = base_query.filter(models.Run.fails >= 1).count()
+    successes = base_query.filter(models.Run.passes > 0,
+                                  models.Run.fails == 0).count()
+    if fails == 0 and successes == 0:
+        return None
+    return (float(fails) / float(successes + fails)) * 100
+
+
 def add_test_run_attachments(attach_dict, test_run_id, session=None):
     """Add attachments a specific test run.
 
@@ -1264,13 +1387,16 @@ def get_runs_by_status_grouped_by_run_metadata(key, start_date=None,
     run_pass_query = session.query(
         sqlalchemy.func.count(models.Run.id), val).filter(
             models.Run.fails == 0, models.Run.passes > 0).join(
-                models.RunMetadata).group_by(val).filter(
-                    models.RunMetadata.key == key)
+                models.RunMetadata,
+                models.Run.id == models.RunMetadata.run_id).group_by(
+                    val).filter(models.RunMetadata.key == key)
     run_fail_query = session.query(
         sqlalchemy.func.count(models.Run.id), val).filter(
             models.Run.fails > 0, models.Run.passes > 0).join(
-                models.RunMetadata).group_by(val).filter(
-                    models.RunMetadata.key == key)
+                models.RunMetadata,
+                models.Run.id == models.RunMetadata.run_id).group_by(
+                    val).filter(
+                        models.RunMetadata.key == key)
 
     run_pass_query = _filter_runs_by_date(run_pass_query, start_date,
                                           stop_date)
@@ -1289,6 +1415,65 @@ def get_runs_by_status_grouped_by_run_metadata(key, start_date=None,
     return result
 
 
+def get_test_runs_by_status_for_run_ids(status, run_ids, key=None,
+                                        session=None):
+    """Get a list of test run dicts by status for all the specified runs
+
+    :param str status: The test status to filter the returned test runs on
+    :param list run_ids: A list of run ids (the uuid column from the runs
+                         table) to get the test runs from
+    :param str key: An optional run_metadata key to add the values for a run
+                    to the output dict for each test_run
+    :param session: optional session object if one isn't provided a new session
+                    will be acquired for the duration of this operation
+
+    :return test_runs: a list of dicts for the test_runs and associated data
+    :rtype: list
+    """
+    session = session or get_session()
+    query = db_utils.model_query(models.TestRun, session).filter(
+        models.TestRun.status == status).join(
+            models.Test, models.TestRun.test_id == models.Test.id).join(
+                models.Run, models.TestRun.run_id == models.Run.id).filter(
+                    models.Run.uuid.in_(run_ids))
+
+    if key:
+        query = query.join(
+            models.RunMetadata,
+            models.TestRun.run_id == models.RunMetadata.run_id).filter(
+                models.RunMetadata.key == key)
+        results = query.values(models.Test.test_id, models.Run.artifacts,
+                               models.TestRun.start_time,
+                               models.TestRun.start_time_microsecond,
+                               models.TestRun.stop_time,
+                               models.TestRun.stop_time_microsecond,
+                               models.RunMetadata.value)
+    else:
+        results = query.values(models.Test.test_id, models.Run.artifacts,
+                               models.TestRun.start_time,
+                               models.TestRun.start_time_microsecond,
+                               models.TestRun.stop_time,
+                               models.TestRun.stop_time_microsecond)
+    test_runs = []
+    for result in results:
+        test_run = {
+            'test_id': result.test_id,
+            'link': result.artifacts,
+            'start_time': result.start_time,
+            'stop_time': result.stop_time,
+        }
+        if result.start_time_microsecond is not None:
+            test_run['start_time'] = test_run['start_time'].replace(
+                microsecond=result.start_time_microsecond)
+        if result.stop_time_microsecond is not None:
+            test_run['stop_time'] = test_run['stop_time'].replace(
+                microsecond=result.stop_time_microsecond)
+        if hasattr(result, "value"):
+            test_run[key] = result.value
+        test_runs.append(test_run)
+    return test_runs
+
+
 def get_all_run_metadata_keys(session=None):
     """Get a list of all the keys used in the run_metadata table
 
